//TypeScript definition for SharePoint 2013 SP.UI

interface Function {
    createDelegate(instance: any, method: () => any): any;
}

interface SharePoint {
    UI: SP_UI;
    Ribbon;
    Utilities;
    ListOperation;
    SOD;
    Guid;
}

interface SP_UI {
    $create_DialogOptions();
    ModalDialog: SP_UI_ModalDialogStatic;
    Notify;
    Status;
    DialogResult: SP_UI_DialogResult;
}

interface SP_UI_ModalDialogOptions {
    /** A string that contains the title of the dialog. */
    title?: string;
    /** 
     * A string that contains the URL of the page that appears in the dialog. 
     * If both url and html are specified, url takes precedence. Either url or html must be specified. 
     */
    url: string;
    /** 
     * A string that contains the HTML of the page that appears in the dialog. 
     * If both html and url are specified, url takes precedence. Either url or html must be specified. 
     */
    html?: string;
    /** An integer value that specifies the x-offset of the dialog. This value works like the CSS left value. */
    x?: number;
    /** An integer value that specifies the y-offset of the dialog. This value works like the CSS top value. */
    y?: number;
    /** 
     * An integer value that specifies the width of the dialog. 
     * If width is not specified, the width of the dialog is autosized by default. 
     * If autosize is false, the width of the dialog is set to 768 pixels. 
     */
    width?: number;
    /**
     * An integer value that specifies the height of the dialog. 
     * If height is not specified, the height of the dialog is autosized by default. If autosize is false, the dialog height is set to 576 pixels.
     */
    height?: number;
    /** A Boolean value that specifies whether the dialog can be maximized. true if the Maximize button is shown; otherwise, false. */
    allowMaximize?: bool;
    /**
     * A Boolean value that specifies whether the dialog opens in a maximized state. true the dialog opens maximized. 
     * Otherwise, the dialog is opened at the requested sized if specified; otherwise, the default size, if specified; otherwise, the autosized size.
     */
    showMaximized?: bool;
    /** A Boolean value that specifies whether the Close button appears on the dialog. */
    showClose?: bool;
    /** A Boolean value that specifies whether the dialog platform handles dialog sizing. */
    autoSize?: bool;
    /** A function pointer that specifies the return callback function. 
      * The function takes two parameters, a dialogResult of type SP.UI.DialogResult Enumeration and a returnValue of type object that contains any data returned by the dialog. */
    dialogReturnValueCallback?: (dialogResult, returnValue) => any;
    /** An object that contains data that are passed to the dialog. */
    args?: any;
}

interface SP_UI_ModalDialogStatic {
    get_childDialog(): HTMLFrameElement;
    /** 
     * Displays a modal dialog with specified dialog options.
     * @param options The options to create the modal dialog.
     */
    showModalDialog(options: SP_UI_ModalDialogOptions): SP_UI_ModalDialog;
    /**
     * Displays a wait screen dialog that does not have a Cancel button using the specified parameters.
     * @param title The title of the wait screen dialog.
     * @param message The message that is shown in the wait screen dialog.
     * @param height The height of the wait screen dialog.
     * @param width The width of the wait screen dialog.
     */
    showWaitScreenWithNoClose(title, message, height, width): SP_UI_ModalDialog;
    /**
     * Displays a wait screen dialog that has a Cancel button using the specified parameters.
     * @param title The title of the wait screen dialog.
     * @param message The message that is shown in the wait screen dialog.
     * @param callbackFunc The callback function that runs when the wait screen dialog is closed.
     * @param height The height of the wait screen dialog.
     * @param width The width of the wait screen dialog.
     */
    showWaitScreenSize(title, message, callbackFunc, height, width): SP_UI_ModalDialog;
    /**
     * Displays a modal dialog with the specified URL, callback function, width, and height.
     * @param url The URL of the page to be shown in the modal dialog.
     * @param callback The callback function that runs when the modal dialog is closed.
     * @param width The width of the modal dialog.
     * @param The height of the modal dialog.
     */
    OpenPopUpPage(url, callback, width, height): void;
    /**
     * Displays a modal dialog using the page at the specified URL.
     * @param url The URL of the page to be shown in the modal dialog.
     */
    ShowPopupDialog(url): void;
    /**
     * Displays a modal dialog with the specified URL, options, callback function, and arguments.
     * @param url The URL of the page to be shown in the modal dialog.
     * @param options The options to create the modal dialog.
     * @param callback The callback function that runs when the modal dialog is closed.
     * @param args The arguments to the modal dialog.
     */
    commonModalDialogOpen(url, options, callback, args): void;
    /**
     * Closes the most recently opened modal dialog with the specified dialog result and return value.
     * @param dialogResult One of the enumeration values specifying the result of the modal dialog.
     * @param returnVal The return value of the modal dialog.
     */
    commonModalDialogClose(dialogResult, returnVal): any;
    /**
     * Refreshes the parent page of the modal dialog when the dialog is closed by clicking OK.
     * @param dialogResult The result of the modal dialog.
     */
    RefreshPage(dialogResult): void;
}

interface SP_UI_ModalDialog {
    /**
     * Closes the most recently opened modal dialog with the specified dialog result.
     * @param dialogResult One of the enumeration values that specifies the result of the modal dialog.
     */
    close(dialogResult): void;
}

interface SP_UI_DialogResult {
    invalid: number;
    cancel: number;
    OK: number;
}

interface SPPageContextInfo {
    webServerRelativeUrl: string;
    webAbsoluteUrl: string;
    siteAbsoluteUrl: string;
    serverRequestPath: string;
    layoutsUrl: string;
    webTitle: string;
    webTemplate: string;
    tenantAppVersion: string;
    webLogoUrl: string;
    webLanguage: number;
    currentLanguage: number;
    currentUICultureName: string;
    currentCultureName: string;
    clientServerTimeDelta: number;
    siteClientTag: string;
    crossDomainPhotosEnabled: bool;
    webUIVersion: number;
    webPermMasks: any;
    pageListId: string;
    pageItemId: number;
    pagePersonalizationScope: number;
    userId: number;
    systemUserKey: string;
    alertsEnabled: bool;
    siteServerRelativeUrl: string;
    allowSilverlightPrompt: string;
};


declare var SP: SharePoint;
declare var Commands;
declare var CUI;
declare var _spPageContextInfo: SPPageContextInfo;
